"use strict";
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const IORedis = require("ioredis");
const serialization_1 = require("./serialization");
const defaultOptions = {
    block: 0,
    count: 1,
    from: '0',
};
const doneWithBacklog = (items) => {
    return items.length === 0;
};
const isGroupAlreadyExistsError = (err) => {
    return err.message.startsWith('BUSYGROUP');
};
const isNoGroupExistsError = (err) => {
    return err.message.startsWith('NOGROUP');
};
const isConnectionClosedError = (err) => {
    return err.message === 'Connection is closed.';
};
const idToDate = (id) => new Date(Number(id.split('-')[0]));
const deserializePair = (pair) => {
    const [id, arr] = pair;
    const [, serialized] = arr;
    const data = serialization_1.deserialize(serialized);
    const paired = [id, data];
    return paired;
};
class Consumer {
    constructor(clientOpts, stream, group, consumer, opts = defaultOptions) {
        this.stream = stream;
        this.group = group;
        this.consumer = consumer;
        this.running = true;
        this.checkBacklog = true;
        this.reading = false;
        this.client = typeof clientOpts === 'string' ? new IORedis(clientOpts) : clientOpts;
        this.block = opts.block || defaultOptions.block;
        this.count = opts.count || defaultOptions.count;
        this.from = opts.from || defaultOptions.from;
    }
    [Symbol.asyncIterator]() {
        return this._iterate();
    }
    disconnect() {
        this.client.disconnect();
    }
    shutdown() {
        this.running = false;
        if (this.reading && this.block === 0) {
            this.disconnect();
        }
    }
    on(name, callback) {
        return this.client.on(name, callback);
    }
    async ack(id) {
        await this.client.xack(this.stream, this.group, id);
    }
    async pending(start = '-', end = '+', length = 100) {
        try {
            const { stream, group } = this;
            const pendingTuples = await this.client.xpending(stream, group, start, end, length);
            const pendingObjects = pendingTuples.map(tuple => {
                const [id, consumer, waiting, deliveries] = tuple;
                const created = idToDate(id);
                const pending = { id, consumer, created, stream, group, deliveries, waiting };
                return pending;
            });
            return pendingObjects;
        }
        catch (err) {
            if (!isNoGroupExistsError(err)) {
                throw err;
            }
            return [];
        }
    }
    async claim(ids) {
        const MIN_IDLE_TIME = 0;
        const pairs = await this.client.xclaim(this.stream, this.group, this.consumer, MIN_IDLE_TIME, ...ids);
        const mapped = pairs.map(pair => deserializePair(pair));
        return mapped;
    }
    async deleteConsumer() {
        await this.client.xgroup('DELCONSUMER', this.stream, this.group, this.consumer);
    }
    async _ensureGroup() {
        try {
            await this.client.xgroup('CREATE', this.stream, this.group, this.from, 'MKSTREAM');
        }
        catch (err) {
            if (!isGroupAlreadyExistsError(err)) {
                throw err;
            }
        }
    }
    async _read(fromId) {
        try {
            this.reading = true;
            const result = await this.client.xreadgroup('GROUP', this.group, this.consumer, 'BLOCK', this.block, 'COUNT', this.count, 'STREAMS', this.stream, fromId);
            if (!result) {
                return null;
            }
            const pairs = result[0][1];
            const mapped = pairs.map(pair => deserializePair(pair));
            return mapped;
        }
        catch (err) {
            if (isConnectionClosedError(err) && !this.running) {
                return null;
            }
            throw err;
        }
        finally {
            this.reading = false;
        }
    }
    _iterate() {
        return __asyncGenerator(this, arguments, function* _iterate_1() {
            yield __await(this._ensureGroup());
            while (this.running) {
                const fromId = this.checkBacklog ? '0-0' : '>';
                const pairs = yield __await(this._read(fromId));
                if (!pairs) {
                    continue;
                }
                if (doneWithBacklog(pairs)) {
                    this.checkBacklog = false;
                }
                for (const pair of pairs) {
                    yield yield __await(pair);
                }
            }
            this.disconnect();
        });
    }
}
exports.Consumer = Consumer;
exports.default = Consumer;
//# sourceMappingURL=Consumer.js.map