"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const IORedis = require("ioredis");
const serialization_1 = require("./serialization");
const defaultOptions = {
    maxLength: 1000,
};
class Writer {
    constructor(clientOpts, stream, opts = defaultOptions) {
        this.stream = stream;
        this.client = typeof clientOpts === 'string' ? new IORedis(clientOpts) : clientOpts;
        this.maxLength = opts.maxLength || defaultOptions.maxLength;
    }
    async write(data) {
        const serialized = serialization_1.serialize(data);
        const id = await this.client.xadd(this.stream, 'MAXLEN', '~', this.maxLength, '*', 'd', serialized);
        return id;
    }
    on(eventName, callback) {
        this.client.on(eventName, callback);
    }
    disconnect() {
        this.client.disconnect();
    }
}
exports.Writer = Writer;
exports.default = Writer;
//# sourceMappingURL=Writer.js.map